@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -37,71 +37,73 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: frm_req_name.c,v 1.10 2003/10/25 15:17:08 tom Exp $")
+MODULE_ID("$Id: frm_req_name.c,v 1.15 2005/04/16 16:59:27 tom Exp $")
 
-static const char *request_names[ MAX_FORM_COMMAND - MIN_FORM_COMMAND + 1 ] = {
-  "NEXT_PAGE"	 ,
-  "PREV_PAGE"	 ,
-  "FIRST_PAGE"	 ,
-  "LAST_PAGE"	 ,
+static const char *request_names[MAX_FORM_COMMAND - MIN_FORM_COMMAND + 1] =
+{
+  "NEXT_PAGE",
+  "PREV_PAGE",
+  "FIRST_PAGE",
+  "LAST_PAGE",
 
-  "NEXT_FIELD"	 ,
-  "PREV_FIELD"	 ,
-  "FIRST_FIELD"	 ,
-  "LAST_FIELD"	 ,
-  "SNEXT_FIELD"	 ,
-  "SPREV_FIELD"	 ,
-  "SFIRST_FIELD" ,
-  "SLAST_FIELD"	 ,
-  "LEFT_FIELD"	 ,
-  "RIGHT_FIELD"	 ,
-  "UP_FIELD"	 ,
-  "DOWN_FIELD"	 ,
+  "NEXT_FIELD",
+  "PREV_FIELD",
+  "FIRST_FIELD",
+  "LAST_FIELD",
+  "SNEXT_FIELD",
+  "SPREV_FIELD",
+  "SFIRST_FIELD",
+  "SLAST_FIELD",
+  "LEFT_FIELD",
+  "RIGHT_FIELD",
+  "UP_FIELD",
+  "DOWN_FIELD",
 
-  "NEXT_CHAR"	 ,
-  "PREV_CHAR"	 ,
-  "NEXT_LINE"	 ,
-  "PREV_LINE"	 ,
-  "NEXT_WORD"	 ,
-  "PREV_WORD"	 ,
-  "BEG_FIELD"	 ,
-  "END_FIELD"	 ,
-  "BEG_LINE"	 ,
-  "END_LINE"	 ,
-  "LEFT_CHAR"	 ,
-  "RIGHT_CHAR"	 ,
-  "UP_CHAR"	 ,
-  "DOWN_CHAR"	 ,
+  "NEXT_CHAR",
+  "PREV_CHAR",
+  "NEXT_LINE",
+  "PREV_LINE",
+  "NEXT_WORD",
+  "PREV_WORD",
+  "BEG_FIELD",
+  "END_FIELD",
+  "BEG_LINE",
+  "END_LINE",
+  "LEFT_CHAR",
+  "RIGHT_CHAR",
+  "UP_CHAR",
+  "DOWN_CHAR",
 
-  "NEW_LINE"	 ,
-  "INS_CHAR"	 ,
-  "INS_LINE"	 ,
-  "DEL_CHAR"	 ,
-  "DEL_PREV"	 ,
-  "DEL_LINE"	 ,
-  "DEL_WORD"	 ,
-  "CLR_EOL"	 ,
-  "CLR_EOF"	 ,
-  "CLR_FIELD"	 ,
-  "OVL_MODE"	 ,
-  "INS_MODE"	 ,
-  "SCR_FLINE"	 ,
-  "SCR_BLINE"	 ,
-  "SCR_FPAGE"	 ,
-  "SCR_BPAGE"	 ,
-  "SCR_FHPAGE"   ,
-  "SCR_BHPAGE"   ,
-  "SCR_FCHAR"    ,
-  "SCR_BCHAR"    ,
-  "SCR_HFLINE"   ,
-  "SCR_HBLINE"   ,
-  "SCR_HFHALF"   ,
-  "SCR_HBHALF"   ,
+  "NEW_LINE",
+  "INS_CHAR",
+  "INS_LINE",
+  "DEL_CHAR",
+  "DEL_PREV",
+  "DEL_LINE",
+  "DEL_WORD",
+  "CLR_EOL",
+  "CLR_EOF",
+  "CLR_FIELD",
+  "OVL_MODE",
+  "INS_MODE",
+  "SCR_FLINE",
+  "SCR_BLINE",
+  "SCR_FPAGE",
+  "SCR_BPAGE",
+  "SCR_FHPAGE",
+  "SCR_BHPAGE",
+  "SCR_FCHAR",
+  "SCR_BCHAR",
+  "SCR_HFLINE",
+  "SCR_HBLINE",
+  "SCR_HFHALF",
+  "SCR_HBHALF",
 
-  "VALIDATION"	 ,
-  "NEXT_CHOICE"	 ,
-  "PREV_CHOICE"	 
+  "VALIDATION",
+  "NEXT_CHOICE",
+  "PREV_CHOICE"
 };
+
 #define A_SIZE (sizeof(request_names)/sizeof(request_names[0]))
 
 /*---------------------------------------------------------------------------
@@ -114,18 +116,19 @@ static const char *request_names[ MAX_FORM_COMMAND - MIN_FORM_COMMAND + 1 ] = {
 |                    NULL                 - on invalid request code
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(const char *)
-form_request_name ( int request )
+form_request_name(int request)
 {
-  if ( (request < MIN_FORM_COMMAND) || (request > MAX_FORM_COMMAND) )
+  T((T_CALLED("form_request_name(%d)"), request));
+
+  if ((request < MIN_FORM_COMMAND) || (request > MAX_FORM_COMMAND))
     {
-      SET_ERROR (E_BAD_ARGUMENT);
-      return (const char *)0;
+      SET_ERROR(E_BAD_ARGUMENT);
+      returnCPtr((const char *)0);
     }
   else
-    return request_names[ request - MIN_FORM_COMMAND ];
+    returnCPtr(request_names[request - MIN_FORM_COMMAND]);
 }
 
-
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
 |   Function      :  int form_request_by_name (const char *str);
@@ -136,28 +139,30 @@ form_request_name ( int request )
 |                    E_NO_MATCH       - request not found
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-form_request_by_name ( const char *str )
-{ 
+form_request_by_name(const char *str)
+{
   /* because the table is so small, it doesn't really hurt
      to run sequentially through it.
-  */
+   */
   unsigned int i = 0;
   char buf[16];
-  
+
+  T((T_CALLED("form_request_by_name(%s)"), _nc_visbuf(str)));
+
   if (str)
     {
-      strncpy(buf,str,sizeof(buf));
-      while( (i<sizeof(buf)) && (buf[i] != '\0') )
+      strncpy(buf, str, sizeof(buf));
+      while ((i < sizeof(buf)) && (buf[i] != '\0'))
 	{
 	  buf[i] = toupper(buf[i]);
 	  i++;
 	}
-      
-      for (i=0; i < A_SIZE; i++)
+
+      for (i = 0; i < A_SIZE; i++)
 	{
-	  if (strncmp(request_names[i],buf,sizeof(buf))==0)
-	    return MIN_FORM_COMMAND + i;
-	} 
+	  if (strncmp(request_names[i], buf, sizeof(buf)) == 0)
+	    returnCode(MIN_FORM_COMMAND + (int) i);
+	}
     }
   RETURN(E_NO_MATCH);
 }
