@@ -13,7 +13,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: fty_ipv4.c,v 1.4 2000/12/09 23:46:12 tom Exp $")
+MODULE_ID("$Id: fty_ipv4.c,v 1.6 2004/05/29 19:13:48 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -26,24 +26,25 @@ MODULE_ID("$Id: fty_ipv4.c,v 1.4 2000/12/09 23:46:12 tom Exp $")
 |   Return Values :  TRUE  - field is valid
 |                    FALSE - field is invalid
 +--------------------------------------------------------------------------*/
-static bool Check_IPV4_Field(FIELD * field, const void * argp GCC_UNUSED)
+static bool
+Check_IPV4_Field(FIELD *field, const void *argp GCC_UNUSED)
 {
-  char *bp = field_buffer(field,0);
+  char *bp = field_buffer(field, 0);
   int num = 0, len;
   unsigned int d1, d2, d3, d4;
 
-  if(isdigit((unsigned char)*bp)) /* Must start with digit */
+  if (isdigit(UChar(*bp)))	/* Must start with digit */
     {
       num = sscanf(bp, "%u.%u.%u.%u%n", &d1, &d2, &d3, &d4, &len);
       if (num == 4)
-        {
-          bp += len;            /* Make bp point to what sscanf() left */
-          while (*bp && isspace((unsigned char)*bp))
-            bp++;               /* Allow trailing whitespace */
-        }
+	{
+	  bp += len;		/* Make bp point to what sscanf() left */
+	  while (*bp && isspace(UChar(*bp)))
+	    bp++;		/* Allow trailing whitespace */
+	}
     }
   return ((num != 4 || *bp || d1 > 255 || d2 > 255
-                           || d3 > 255 || d4 > 255) ? FALSE : TRUE);
+	   || d3 > 255 || d4 > 255) ? FALSE : TRUE);
 }
 
 /*---------------------------------------------------------------------------
@@ -57,14 +58,16 @@ static bool Check_IPV4_Field(FIELD * field, const void * argp GCC_UNUSED)
 |   Return Values :  TRUE  - character is valid
 |                    FALSE - character is invalid
 +--------------------------------------------------------------------------*/
-static bool Check_IPV4_Character(int c, const void * argp GCC_UNUSED)
+static bool
+Check_IPV4_Character(int c, const void *argp GCC_UNUSED)
 {
-  return ((isdigit(c) || (c=='.')) ? TRUE : FALSE);
+  return ((isdigit(UChar(c)) || (c == '.')) ? TRUE : FALSE);
 }
 
-static FIELDTYPE typeIPV4 = {
+static FIELDTYPE typeIPV4 =
+{
   _RESIDENT,
-  1,                           /* this is mutable, so we can't be const */
+  1,				/* this is mutable, so we can't be const */
   (FIELDTYPE *)0,
   (FIELDTYPE *)0,
   NULL,
