@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2003,2004 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -37,7 +37,7 @@
 
 #include "menu.priv.h"
 
-MODULE_ID("$Id: m_new.c,v 1.13 2003/10/25 14:54:48 tom Exp $")
+MODULE_ID("$Id: m_new.c,v 1.16 2004/12/25 21:57:53 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnmenu  
@@ -51,10 +51,11 @@ MODULE_ID("$Id: m_new.c,v 1.13 2003/10/25 14:54:48 tom Exp $")
 |   Return Values :  NULL on error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(MENU *)
-new_menu (ITEM ** items)
+new_menu(ITEM ** items)
 {
-  MENU *menu = (MENU *)calloc(1,sizeof(MENU));
-  
+  MENU *menu = (MENU *) calloc(1, sizeof(MENU));
+
+  T((T_CALLED("new_menu(%p)"), items));
   if (menu)
     {
       *menu = _nc_Default_Menu;
@@ -63,10 +64,10 @@ new_menu (ITEM ** items)
       menu->cols = menu->fcols;
       if (items && *items)
 	{
-	  if (!_nc_Connect_Items(menu,items))
+	  if (!_nc_Connect_Items(menu, items))
 	    {
 	      free(menu);
-	      menu = (MENU *)0;
+	      menu = (MENU *) 0;
 	    }
 	}
     }
@@ -74,7 +75,7 @@ new_menu (ITEM ** items)
   if (!menu)
     SET_ERROR(E_SYSTEM_ERROR);
 
-  return(menu);
+  returnMenu(menu);
 }
 
 /*---------------------------------------------------------------------------
@@ -89,17 +90,18 @@ new_menu (ITEM ** items)
 |                    E_POSTED           - Menu is already posted
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-free_menu (MENU * menu)
+free_menu(MENU * menu)
 {
+  T((T_CALLED("free_menu(%p)"), menu));
   if (!menu)
     RETURN(E_BAD_ARGUMENT);
-  
-  if ( menu->status & _POSTED )
+
+  if (menu->status & _POSTED)
     RETURN(E_POSTED);
-  
-  if (menu->items) 
+
+  if (menu->items)
     _nc_Disconnect_Items(menu);
-  
+
   if ((menu->status & _MARK_ALLOCATED) && menu->mark)
     free(menu->mark);
 
