@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2003,2004 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -37,7 +37,7 @@
 
 #include "menu.priv.h"
 
-MODULE_ID("$Id: m_opts.c,v 1.16 2003/11/08 20:50:19 tom Exp $")
+MODULE_ID("$Id: m_opts.c,v 1.19 2004/12/25 21:36:12 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnmenu
@@ -53,8 +53,10 @@ MODULE_ID("$Id: m_opts.c,v 1.16 2003/11/08 20:50:19 tom Exp $")
 |                    E_POSTED       - menu is already posted
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_menu_opts (MENU * menu, Menu_Options opts)
+set_menu_opts(MENU * menu, Menu_Options opts)
 {
+  T((T_CALLED("set_menu_opts(%p,%d)"), menu, opts));
+
   opts &= ALL_MENU_OPTS;
 
   if (opts & ~ALL_MENU_OPTS)
@@ -62,18 +64,18 @@ set_menu_opts (MENU * menu, Menu_Options opts)
 
   if (menu)
     {
-      if ( menu->status & _POSTED )
+      if (menu->status & _POSTED)
 	RETURN(E_POSTED);
 
-      if ( (opts&O_ROWMAJOR) != (menu->opt&O_ROWMAJOR))
+      if ((opts & O_ROWMAJOR) != (menu->opt & O_ROWMAJOR))
 	{
 	  /* we need this only if the layout really changed ... */
 	  if (menu->items && menu->items[0])
 	    {
-	      menu->toprow  = 0;
+	      menu->toprow = 0;
 	      menu->curitem = menu->items[0];
 	      assert(menu->curitem);
-	      set_menu_format( menu, menu->frows, menu->fcols );
+	      set_menu_format(menu, menu->frows, menu->fcols);
 	    }
 	}
 
@@ -83,13 +85,13 @@ set_menu_opts (MENU * menu, Menu_Options opts)
 	{
 	  ITEM **item;
 
-	  if ( ((item=menu->items) != (ITEM**)0) )
-	    for(;*item;item++)
+	  if (((item = menu->items) != (ITEM **) 0))
+	    for (; *item; item++)
 	      (*item)->value = FALSE;
 	}
 
       if (opts & O_SHOWDESC)	/* this also changes the geometry */
-	_nc_Calculate_Item_Length_and_Width( menu );
+	_nc_Calculate_Item_Length_and_Width(menu);
     }
   else
     _nc_Default_Menu.opt = opts;
@@ -111,10 +113,13 @@ set_menu_opts (MENU * menu, Menu_Options opts)
 |                    E_POSTED       - menu is already posted
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-menu_opts_off (MENU *menu, Menu_Options  opts)
+menu_opts_off(MENU * menu, Menu_Options opts)
 {
-  MENU *cmenu = menu; /* use a copy because set_menu_opts must detect
-                         NULL menu itself to adjust its behavior */
+  MENU *cmenu = menu;		/* use a copy because set_menu_opts must detect
+
+				   NULL menu itself to adjust its behavior */
+
+  T((T_CALLED("menu_opts_off(%p,%d)"), menu, opts));
 
   opts &= ALL_MENU_OPTS;
   if (opts & ~ALL_MENU_OPTS)
@@ -123,7 +128,7 @@ menu_opts_off (MENU *menu, Menu_Options  opts)
     {
       Normalize_Menu(cmenu);
       opts = cmenu->opt & ~opts;
-      return set_menu_opts( menu, opts );
+      returnCode(set_menu_opts(menu, opts));
     }
 }
 
@@ -141,10 +146,13 @@ menu_opts_off (MENU *menu, Menu_Options  opts)
 |                    E_POSTED       - menu is already posted
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-menu_opts_on (MENU * menu, Menu_Options opts)
+menu_opts_on(MENU * menu, Menu_Options opts)
 {
-  MENU *cmenu = menu; /* use a copy because set_menu_opts must detect
-                         NULL menu itself to adjust its behavior */
+  MENU *cmenu = menu;		/* use a copy because set_menu_opts must detect
+
+				   NULL menu itself to adjust its behavior */
+
+  T((T_CALLED("menu_opts_on(%p,%d)"), menu, opts));
 
   opts &= ALL_MENU_OPTS;
   if (opts & ~ALL_MENU_OPTS)
@@ -153,7 +161,7 @@ menu_opts_on (MENU * menu, Menu_Options opts)
     {
       Normalize_Menu(cmenu);
       opts = cmenu->opt | opts;
-      return set_menu_opts(menu, opts);
+      returnCode(set_menu_opts(menu, opts));
     }
 }
 
@@ -166,9 +174,10 @@ menu_opts_on (MENU * menu, Menu_Options opts)
 |   Return Values :  Menu options
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(Menu_Options)
-menu_opts (const MENU *menu)
+menu_opts(const MENU * menu)
 {
-  return (ALL_MENU_OPTS & Normalize_Menu( menu )->opt);
+  T((T_CALLED("menu_opts(%p)"), menu));
+  returnMenuOpts(ALL_MENU_OPTS & Normalize_Menu(menu)->opt);
 }
 
 /* m_opts.c ends here */
