@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2002,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,9 +29,9 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996-on                 *
  ****************************************************************************/
 
-
 /*
  *	comp_error.c -- Error message routines
  *
@@ -41,92 +41,113 @@
 
 #include <tic.h>
 
-MODULE_ID("$Id: comp_error.c,v 1.16 1998/08/01 23:39:51 tom Exp $")
+MODULE_ID("$Id: comp_error.c,v 1.29 2005/08/20 19:22:36 tom Exp $")
 
-bool	_nc_suppress_warnings;
-int	_nc_curr_line;		/* current line # in input */
-int	_nc_curr_col;		/* current column # in input */
+NCURSES_EXPORT_VAR(bool) _nc_suppress_warnings = FALSE;
+NCURSES_EXPORT_VAR(int) _nc_curr_line = 0; /* current line # in input */
+NCURSES_EXPORT_VAR(int) _nc_curr_col = 0; /* current column # in input */
 
 static const char *sourcename;
-static char termtype[MAX_NAME_SIZE+1];
+static char *termtype;
 
-void _nc_set_source(const char *const name)
+NCURSES_EXPORT(const char *)
+_nc_get_source(void)
 {
-	sourcename = name;
+    return sourcename;
 }
 
-void _nc_set_type(const char *const name)
+NCURSES_EXPORT(void)
+_nc_set_source(const char *const name)
 {
+    sourcename = name;
+}
+
+NCURSES_EXPORT(void)
+_nc_set_type(const char *const name)
+{
+    if (termtype == 0)
+	termtype = typeMalloc(char, MAX_NAME_SIZE + 1);
+    if (termtype != 0) {
+	termtype[0] = '\0';
 	if (name)
-		strncpy( termtype, name, MAX_NAME_SIZE );
-	else
-		termtype[0] = '\0';
+	    strncat(termtype, name, MAX_NAME_SIZE);
+    }
 }
 
-void _nc_get_type(char *name)
+NCURSES_EXPORT(void)
+_nc_get_type(char *name)
 {
-	strcpy( name, termtype );
+#if NO_LEAKS
+    if (name == 0 && termtype != 0) {
+	FreeAndNull(termtype);
+	return;
+    }
+#endif
+    if (name != 0)
+	strcpy(name, termtype != 0 ? termtype : "");
 }
 
-static inline void where_is_problem(void)
+static inline void
+where_is_problem(void)
 {
-	fprintf (stderr, "\"%s\"", sourcename);
-	if (_nc_curr_line >= 0)
-		fprintf (stderr, ", line %d", _nc_curr_line);
-	if (_nc_curr_col >= 0)
-		fprintf (stderr, ", col %d", _nc_curr_col);
-	if (termtype[0])
-		fprintf (stderr, ", terminal '%s'", termtype);
-	fputc(':', stderr);
-	fputc(' ', stderr);
+    fprintf(stderr, "\"%s\"", sourcename);
+    if (_nc_curr_line >= 0)
+	fprintf(stderr, ", line %d", _nc_curr_line);
+    if (_nc_curr_col >= 0)
+	fprintf(stderr, ", col %d", _nc_curr_col);
+    if (termtype != 0 && termtype[0] != '\0')
+	fprintf(stderr, ", terminal '%s'", termtype);
+    fputc(':', stderr);
+    fputc(' ', stderr);
 }
 
-void _nc_warning(const char *const fmt, ...)
+NCURSES_EXPORT(void)
+_nc_warning(const char *const fmt,...)
 {
-va_list argp;
+    va_list argp;
 
-	if (_nc_suppress_warnings)
-	    return;
+    if (_nc_suppress_warnings)
+	return;
 
-	where_is_problem();
-	va_start(argp,fmt);
-	vfprintf (stderr, fmt, argp);
-	fprintf (stderr, "\n");
-	va_end(argp);
+    where_is_problem();
+    va_start(argp, fmt);
+    vfprintf(stderr, fmt, argp);
+    fprintf(stderr, "\n");
+    va_end(argp);
 }
 
-
-void _nc_err_abort(const char *const fmt, ...)
+NCURSES_EXPORT(void)
+_nc_err_abort(const char *const fmt,...)
 {
-va_list argp;
+    va_list argp;
 
-	where_is_problem();
-	va_start(argp,fmt);
-	vfprintf (stderr, fmt, argp);
-	fprintf (stderr, "\n");
-	va_end(argp);
-	exit(EXIT_FAILURE);
+    where_is_problem();
+    va_start(argp, fmt);
+    vfprintf(stderr, fmt, argp);
+    fprintf(stderr, "\n");
+    va_end(argp);
+    exit(EXIT_FAILURE);
 }
 
-
-void _nc_syserr_abort(const char *const fmt, ...)
+NCURSES_EXPORT(void)
+_nc_syserr_abort(const char *const fmt,...)
 {
-va_list argp;
+    va_list argp;
 
-	where_is_problem();
-	va_start(argp,fmt);
-	vfprintf (stderr, fmt, argp);
-	fprintf (stderr, "\n");
-	va_end(argp);
+    where_is_problem();
+    va_start(argp, fmt);
+    vfprintf(stderr, fmt, argp);
+    fprintf(stderr, "\n");
+    va_end(argp);
 
-	/* If we're debugging, try to show where the problem occurred - this
-	 * will dump core.
-	 */
+    /* If we're debugging, try to show where the problem occurred - this
+     * will dump core.
+     */
 #if defined(TRACE) || !defined(NDEBUG)
-	abort();
+    abort();
 #else
-	/* Dumping core in production code is not a good idea.
-	 */
-	exit(EXIT_FAILURE);
+    /* Dumping core in production code is not a good idea.
+     */
+    exit(EXIT_FAILURE);
 #endif
 }
