@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
+ * Copyright (c) 1998-2004,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas E. Dickey                        1996 on                 *
  ****************************************************************************/
 
 /*
@@ -50,7 +51,7 @@
 #include <term_entry.h>
 #include <tic.h>
 
-MODULE_ID("$Id: comp_scan.c,v 1.59 2001/09/23 00:56:29 tom Exp $")
+MODULE_ID("$Id: comp_scan.c,v 1.76 2005/06/04 22:04:45 tom Exp $")
 
 /*
  * Maximum length of string capability we'll accept before raising an error.
@@ -84,6 +85,7 @@ _nc_curr_token =
  *****************************************************************************/
 
 static bool first_column;	/* See 'next_char()' below */
+static bool had_newline;
 static char separator;		/* capability separator */
 static int pushtype;		/* type of pushback token */
 static char *pushname;
@@ -93,11 +95,193 @@ NCURSES_EXPORT_VAR(bool)
 _nc_disable_period = FALSE;	/* used by tic -a option */
 #endif
 
-static int last_char(void);
-static int next_char(void);
-static long stream_pos(void);
-static bool end_of_stream(void);
-static void push_back(char c);
+/*****************************************************************************
+ *
+ * Character-stream handling
+ *
+ *****************************************************************************/
+
+#define LEXBUFSIZ	1024
+
+static char *bufptr;		/* otherwise, the input buffer pointer */
+static char *bufstart;		/* start of buffer so we can compute offsets */
+static FILE *yyin;		/* scanner's input file descriptor */
+
+/*
+ *	_nc_reset_input()
+ *
+ *	Resets the input-reading routines.  Used on initialization,
+ *	or after a seek has been done.  Exactly one argument must be
+ *	non-null.
+ */
+
+NCURSES_EXPORT(void)
+_nc_reset_input(FILE *fp, char *buf)
+{
+    pushtype = NO_PUSHBACK;
+    if (pushname != 0)
+	pushname[0] = '\0';
+    yyin = fp;
+    bufstart = bufptr = buf;
+    _nc_curr_file_pos = 0L;
+    if (fp != 0)
+	_nc_curr_line = 0;
+    _nc_curr_col = 0;
+}
+
+/*
+ *	int last_char()
+ *
+ *	Returns the final nonblank character on the current input buffer
+ */
+static int
+last_char(void)
+{
+    size_t len = strlen(bufptr);
+    while (len--) {
+	if (!isspace(UChar(bufptr[len])))
+	    return bufptr[len];
+    }
+    return 0;
+}
+
+/*
+ *	int next_char()
+ *
+ *	Returns the next character in the input stream.  Comments and leading
+ *	white space are stripped.
+ *
+ *	The global state variable 'firstcolumn' is set TRUE if the character
+ *	returned is from the first column of the input line.
+ *
+ *	The global variable _nc_curr_line is incremented for each new line.
+ *	The global variable _nc_curr_file_pos is set to the file offset of the
+ *	beginning of each line.
+ */
+
+static int
+next_char(void)
+{
+    static char *result;
+    static size_t allocated;
+    int the_char;
+
+    if (!yyin) {
+	if (result != 0) {
+	    FreeAndNull(result);
+	    FreeAndNull(pushname);
+	    allocated = 0;
+	}
+	/*
+	 * An string with an embedded null will truncate the input.  This is
+	 * intentional (we don't read binary files here).
+	 */
+	if (bufptr == 0 || *bufptr == '\0')
+	    return (EOF);
+	if (*bufptr == '\n') {
+	    _nc_curr_line++;
+	    _nc_curr_col = 0;
+	} else if (*bufptr == '\t') {
+	    _nc_curr_col = (_nc_curr_col | 7);
+	}
+    } else if (!bufptr || !*bufptr) {
+	/*
+	 * In theory this could be recoded to do its I/O one character at a
+	 * time, saving the buffer space.  In practice, this turns out to be
+	 * quite hard to get completely right.  Try it and see.  If you
+	 * succeed, don't forget to hack push_back() correspondingly.
+	 */
+	size_t used;
+	size_t len;
+
+	do {
+	    bufstart = 0;
+	    used = 0;
+	    do {
+		if (used + (LEXBUFSIZ / 4) >= allocated) {
+		    allocated += (allocated + LEXBUFSIZ);
+		    result = typeRealloc(char, allocated, result);
+		    if (result == 0)
+			return (EOF);
+		}
+		if (used == 0)
+		    _nc_curr_file_pos = ftell(yyin);
+
+		if (fgets(result + used, (int) (allocated - used), yyin) != 0) {
+		    bufstart = result;
+		    if (used == 0) {
+			_nc_curr_line++;
+			_nc_curr_col = 0;
+		    }
+		} else {
+		    if (used != 0)
+			strcat(result, "\n");
+		}
+		if ((bufptr = bufstart) != 0) {
+		    used = strlen(bufptr);
+		    while (iswhite(*bufptr)) {
+			if (*bufptr == '\t') {
+			    _nc_curr_col = (_nc_curr_col | 7) + 1;
+			} else {
+			    _nc_curr_col++;
+			}
+			bufptr++;
+		    }
+
+		    /*
+		     * Treat a trailing <cr><lf> the same as a <newline> so we
+		     * can read files on OS/2, etc.
+		     */
+		    if ((len = strlen(bufptr)) > 1) {
+			if (bufptr[len - 1] == '\n'
+			    && bufptr[len - 2] == '\r') {
+			    len--;
+			    bufptr[len - 1] = '\n';
+			    bufptr[len] = '\0';
+			}
+		    }
+		} else {
+		    return (EOF);
+		}
+	    } while (bufptr[len - 1] != '\n');	/* complete a line */
+	} while (result[0] == '#');	/* ignore comments */
+    } else if (*bufptr == '\t') {
+	_nc_curr_col = (_nc_curr_col | 7);
+    }
+
+    first_column = (bufptr == bufstart);
+    if (first_column)
+	had_newline = FALSE;
+
+    _nc_curr_col++;
+    the_char = *bufptr++;
+    return UChar(the_char);
+}
+
+static void
+push_back(char c)
+/* push a character back onto the input stream */
+{
+    if (bufptr == bufstart)
+	_nc_syserr_abort("Can't backspace off beginning of line");
+    *--bufptr = c;
+    _nc_curr_col--;
+}
+
+static long
+stream_pos(void)
+/* return our current character position in the input stream */
+{
+    return (yyin ? ftell(yyin) : (bufptr ? bufptr - bufstart : 0));
+}
+
+static bool
+end_of_stream(void)
+/* are we at end of input? */
+{
+    return ((yyin ? feof(yyin) : (bufptr && *bufptr == '\0'))
+	    ? TRUE : FALSE);
+}
 
 /* Assume we may be looking at a termcap-style continuation */
 static inline int
@@ -148,8 +332,11 @@ _nc_get_token(bool silent)
     static const char terminfo_punct[] = "@%&*!#";
     static char *buffer;
 
+    char *after_list;
+    char *after_name;
     char *numchk;
     char *ptr;
+    char *s;
     char numbuf[80];
     int ch;
     int dot_flag = FALSE;
@@ -157,6 +344,10 @@ _nc_get_token(bool silent)
     long number;
     long token_start;
     unsigned found;
+#ifdef TRACE
+    int old_line;
+    int old_col;
+#endif
 
     if (pushtype != NO_PUSHBACK) {
 	int retval = pushtype;
@@ -173,16 +364,31 @@ _nc_get_token(bool silent)
 	return (retval);
     }
 
-    if (end_of_stream())
+    if (end_of_stream()) {
+	yyin = 0;
+	next_char();		/* frees its allocated memory */
+	if (buffer != 0) {
+	    if (_nc_curr_token.tk_name == buffer)
+		_nc_curr_token.tk_name = 0;
+	    FreeAndNull(buffer);
+	}
 	return (EOF);
+    }
 
   start_token:
     token_start = stream_pos();
-    while ((ch = next_char()) == '\n' || iswhite(ch))
+    while ((ch = next_char()) == '\n' || iswhite(ch)) {
+	if (ch == '\n')
+	    had_newline = TRUE;
 	continue;
+    }
 
     ch = eat_escaped_newline(ch);
 
+#ifdef TRACE
+    old_line = _nc_curr_line;
+    old_col = _nc_curr_col;
+#endif
     if (ch == EOF)
 	type = EOF;
     else {
@@ -208,36 +414,44 @@ _nc_get_token(bool silent)
 	}
 
 	/* have to make some punctuation chars legal for terminfo */
-	if (!isalnum(ch)
+	if (!isalnum(UChar(ch))
 #if NCURSES_EXT_FUNCS
 	    && !(ch == '.' && _nc_disable_period)
 #endif
 	    && !strchr(terminfo_punct, (char) ch)) {
 	    if (!silent)
-		_nc_warning("Illegal character (expected alphanumeric or %s) - %s",
+		_nc_warning("Illegal character (expected alphanumeric or %s) - '%s'",
 			    terminfo_punct, unctrl((chtype) ch));
 	    _nc_panic_mode(separator);
 	    goto start_token;
 	}
 
 	if (buffer == 0)
-	    buffer = _nc_doalloc(buffer, MAX_ENTRY_SIZE);
+	    buffer = typeMalloc(char, MAX_ENTRY_SIZE);
 
+#ifdef TRACE
+	old_line = _nc_curr_line;
+	old_col = _nc_curr_col;
+#endif
 	ptr = buffer;
 	*(ptr++) = ch;
 
 	if (first_column) {
-	    char *desc;
-
 	    _nc_comment_start = token_start;
 	    _nc_comment_end = _nc_curr_file_pos;
 	    _nc_start_line = _nc_curr_line;
 
 	    _nc_syntax = ERR;
+	    after_name = 0;
+	    after_list = 0;
 	    while ((ch = next_char()) != '\n') {
-		if (ch == EOF)
-		    _nc_err_abort("premature EOF");
-		else if (ch == ':' && last_char() != ',') {
+		if (ch == EOF) {
+		    _nc_err_abort(MSG_NO_INPUTS);
+		} else if (ch == '|') {
+		    after_list = ptr;
+		    if (after_name == 0)
+			after_name = ptr;
+		} else if (ch == ':' && last_char() != ',') {
 		    _nc_syntax = SYN_TERMCAP;
 		    separator = ':';
 		    break;
@@ -245,14 +459,18 @@ _nc_get_token(bool silent)
 		    _nc_syntax = SYN_TERMINFO;
 		    separator = ',';
 		    /*
-		     * Fall-through here is not an accident.  The idea is that
-		     * if we see a comma, we figure this is terminfo unless we
-		     * subsequently run into a colon -- but we don't stop
-		     * looking for that colon until hitting a newline.  This
+		     * If we did not see a '|', then we found a name with no
+		     * aliases or description.
+		     */
+		    if (after_name == 0)
+			break;
+		    /*
+		     * If we see a comma, we assume this is terminfo unless we
+		     * subsequently run into a colon.  But we don't stop
+		     * looking for a colon until hitting a newline.  This
 		     * allows commas to be embedded in description fields of
 		     * either syntax.
 		     */
-		    /* FALLTHRU */
 		} else
 		    ch = eat_escaped_newline(ch);
 
@@ -277,58 +495,62 @@ _nc_get_token(bool silent)
 
 	    /*
 	     * This is the soonest we have the terminal name fetched.  Set up
-	     * for following warning messages.
+	     * for following warning messages.  If there's no '|', then there
+	     * is no description.
 	     */
-	    ptr = strchr(buffer, '|');
-	    if (ptr == (char *) NULL)
-		ptr = buffer + strlen(buffer);
-	    ch = *ptr;
-	    *ptr = '\0';
-	    _nc_set_type(buffer);
-	    *ptr = ch;
+	    if (after_name != 0) {
+		ch = *after_name;
+		*after_name = '\0';
+		_nc_set_type(buffer);
+		*after_name = ch;
+	    }
 
 	    /*
 	     * Compute the boundary between the aliases and the description
 	     * field for syntax-checking purposes.
 	     */
-	    desc = strrchr(buffer, '|');
-	    if (!silent && desc) {
-		if (*desc == '\0')
-		    _nc_warning("empty longname field");
-		else if (strchr(desc, ' ') == (char *) NULL)
-		    _nc_warning("older tic versions may treat the description field as an alias");
+	    if (after_list != 0) {
+		if (!silent) {
+		    if (*after_list == '\0')
+			_nc_warning("empty longname field");
+		    else if (strchr(after_list, ' ') == 0)
+			_nc_warning("older tic versions may treat the description field as an alias");
+		}
+	    } else {
+		after_list = buffer + strlen(buffer);
+		DEBUG(1, ("missing description"));
 	    }
-	    if (!desc)
-		desc = buffer + strlen(buffer);
 
 	    /*
 	     * Whitespace in a name field other than the long name can confuse
 	     * rdist and some termcap tools.  Slashes are a no-no.  Other
 	     * special characters can be dangerous due to shell expansion.
 	     */
-	    for (ptr = buffer; ptr < desc; ptr++) {
-		if (isspace(UChar(*ptr))) {
+	    for (s = buffer; s < after_list; ++s) {
+		if (isspace(UChar(*s))) {
 		    if (!silent)
 			_nc_warning("whitespace in name or alias field");
 		    break;
-		} else if (*ptr == '/') {
+		} else if (*s == '/') {
 		    if (!silent)
 			_nc_warning("slashes aren't allowed in names or aliases");
 		    break;
-		} else if (strchr("$[]!*?", *ptr)) {
+		} else if (strchr("$[]!*?", *s)) {
 		    if (!silent)
-			_nc_warning("dubious character `%c' in name or alias field", *ptr);
+			_nc_warning("dubious character `%c' in name or alias field", *s);
 		    break;
 		}
 	    }
 
-	    ptr = buffer;
-
 	    _nc_curr_token.tk_name = buffer;
 	    type = NAMES;
 	} else {
+	    if (had_newline && _nc_syntax == SYN_TERMCAP) {
+		_nc_warning("Missing backslash before newline");
+		had_newline = FALSE;
+	    }
 	    while ((ch = next_char()) != EOF) {
-		if (!isalnum(ch)) {
+		if (!isalnum(UChar(ch))) {
 		    if (_nc_syntax == SYN_TERMINFO) {
 			if (ch != '_')
 			    break;
@@ -393,7 +615,7 @@ _nc_get_token(bool silent)
 		/* just to get rid of the compiler warning */
 		type = UNDEF;
 		if (!silent)
-		    _nc_warning("Illegal character - %s", unctrl((chtype) ch));
+		    _nc_warning("Illegal character - '%s'", unctrl((chtype) ch));
 	    }
 	}			/* end else (first_column == FALSE) */
     }				/* end else (ch != EOF) */
@@ -404,6 +626,11 @@ _nc_get_token(bool silent)
     if (dot_flag == TRUE)
 	DEBUG(8, ("Commented out "));
 
+    if (_nc_tracing >= DEBUG_LEVEL(8)) {
+	_tracef("parsed %d.%d to %d.%d",
+		old_line, old_col,
+		_nc_curr_line, _nc_curr_col);
+    }
     if (_nc_tracing >= DEBUG_LEVEL(7)) {
 	switch (type) {
 	case BOOLEAN:
@@ -448,8 +675,9 @@ _nc_get_token(bool silent)
 	type = _nc_get_token(silent);
 
     DEBUG(3, ("token: `%s', class %d",
-	      _nc_curr_token.tk_name != 0 ? _nc_curr_token.tk_name :
-	      "<null>",
+	      ((_nc_curr_token.tk_name != 0)
+	       ? _nc_curr_token.tk_name
+	       : "<null>"),
 	      type));
 
     return (type);
@@ -474,7 +702,7 @@ _nc_get_token(bool silent)
  *
  */
 
-NCURSES_EXPORT(char)
+NCURSES_EXPORT(int)
 _nc_trans_string(char *ptr, char *last)
 {
     int count = 0;
@@ -492,10 +720,10 @@ _nc_trans_string(char *ptr, char *last)
 	if (ch == '^' && last_ch != '%') {
 	    ch = c = next_char();
 	    if (c == EOF)
-		_nc_err_abort("Premature EOF");
+		_nc_err_abort(MSG_NO_INPUTS);
 
 	    if (!(is7bits(ch) && isprint(ch))) {
-		_nc_warning("Illegal ^ character - %s", unctrl(ch));
+		_nc_warning("Illegal ^ character - '%s'", unctrl(ch));
 	    }
 	    if (ch == '?') {
 		*(ptr++) = '\177';
@@ -509,14 +737,14 @@ _nc_trans_string(char *ptr, char *last)
 	} else if (ch == '\\') {
 	    ch = c = next_char();
 	    if (c == EOF)
-		_nc_err_abort("Premature EOF");
+		_nc_err_abort(MSG_NO_INPUTS);
 
 	    if (ch >= '0' && ch <= '7') {
 		number = ch - '0';
 		for (i = 0; i < 2; i++) {
 		    ch = c = next_char();
 		    if (c == EOF)
-			_nc_err_abort("Premature EOF");
+			_nc_err_abort(MSG_NO_INPUTS);
 
 		    if (c < '0' || c > '7') {
 			if (isdigit(c)) {
@@ -590,21 +818,31 @@ _nc_trans_string(char *ptr, char *last)
 		    continue;
 
 		default:
-		    _nc_warning("Illegal character %s in \\ sequence",
+		    _nc_warning("Illegal character '%s' in \\ sequence",
 				unctrl(ch));
+		    /* FALLTHRU */
+		case '|':
 		    *(ptr++) = (char) ch;
 		}		/* endswitch (ch) */
 	    }			/* endelse (ch < '0' ||  ch > '7') */
 	}
 	/* end else if (ch == '\\') */
 	else if (ch == '\n' && (_nc_syntax == SYN_TERMINFO)) {
-	    /* newlines embedded in a terminfo string are ignored */
+	    /*
+	     * Newlines embedded in a terminfo string are ignored, provided
+	     * that the next line begins with whitespace.
+	     */
 	    ignored = TRUE;
 	} else {
 	    *(ptr++) = (char) ch;
 	}
 
 	if (!ignored) {
+	    if (_nc_curr_col <= 1) {
+		push_back(ch);
+		ch = '\n';
+		break;
+	    }
 	    last_ch = ch;
 	    count++;
 	}
@@ -639,11 +877,14 @@ _nc_push_token(int tokclass)
      */
     pushtype = tokclass;
     if (pushname == 0)
-	pushname = _nc_doalloc(pushname, MAX_NAME_SIZE + 1);
+	pushname = typeMalloc(char, MAX_NAME_SIZE + 1);
     _nc_get_type(pushname);
 
     DEBUG(3, ("pushing token: `%s', class %d",
-	      _nc_curr_token.tk_name, pushtype));
+	      ((_nc_curr_token.tk_name != 0)
+	       ? _nc_curr_token.tk_name
+	       : "<null>"),
+	      pushtype));
 }
 
 /*
@@ -663,169 +904,10 @@ _nc_panic_mode(char ch)
     }
 }
 
-/*****************************************************************************
- *
- * Character-stream handling
- *
- *****************************************************************************/
-
-#define LEXBUFSIZ	1024
-
-static char *bufptr;		/* otherwise, the input buffer pointer */
-static char *bufstart;		/* start of buffer so we can compute offsets */
-static FILE *yyin;		/* scanner's input file descriptor */
-
-/*
- *	_nc_reset_input()
- *
- *	Resets the input-reading routines.  Used on initialization,
- *	or after a seek has been done.  Exactly one argument must be
- *	non-null.
- */
-
+#if NO_LEAKS
 NCURSES_EXPORT(void)
-_nc_reset_input(FILE * fp, char *buf)
+_nc_comp_scan_leaks(void)
 {
-    pushtype = NO_PUSHBACK;
-    if (pushname != 0)
-	pushname[0] = '\0';
-    yyin = fp;
-    bufstart = bufptr = buf;
-    _nc_curr_file_pos = 0L;
-    if (fp != 0)
-	_nc_curr_line = 0;
-    _nc_curr_col = 0;
-}
-
-/*
- *	int last_char()
- *
- *	Returns the final nonblank character on the current input buffer
- */
-static int
-last_char(void)
-{
-    size_t len = strlen(bufptr);
-    while (len--) {
-	if (!isspace(UChar(bufptr[len])))
-	    return bufptr[len];
-    }
-    return 0;
-}
-
-/*
- *	int next_char()
- *
- *	Returns the next character in the input stream.  Comments and leading
- *	white space are stripped.
- *
- *	The global state variable 'firstcolumn' is set TRUE if the character
- *	returned is from the first column of the input line.
- *
- *	The global variable _nc_curr_line is incremented for each new line.
- *	The global variable _nc_curr_file_pos is set to the file offset of the
- *	beginning of each line.
- */
-
-static int
-next_char(void)
-{
-    if (!yyin) {
-	/*
-	 * An string with an embedded null will truncate the input.  This is
-	 * intentional (we don't read binary files here).
-	 */
-	if (*bufptr == '\0')
-	    return (EOF);
-	if (*bufptr == '\n') {
-	    _nc_curr_line++;
-	    _nc_curr_col = 0;
-	}
-    } else if (!bufptr || !*bufptr) {
-	/*
-	 * In theory this could be recoded to do its I/O one character at a
-	 * time, saving the buffer space.  In practice, this turns out to be
-	 * quite hard to get completely right.  Try it and see.  If you
-	 * succeed, don't forget to hack push_back() correspondingly.
-	 */
-	static char *result;
-	static size_t allocated;
-	size_t used;
-	size_t len;
-
-	do {
-	    bufstart = 0;
-	    used = 0;
-	    do {
-		if (used + (LEXBUFSIZ / 4) >= allocated) {
-		    allocated += (allocated + LEXBUFSIZ);
-		    result = _nc_doalloc(result, allocated);
-		    if (result == 0)
-			return (EOF);
-		}
-		if (used == 0)
-		    _nc_curr_file_pos = ftell(yyin);
-
-		if (fgets(result + used, allocated - used, yyin) != NULL) {
-		    bufstart = result;
-		    if (used == 0) {
-			_nc_curr_line++;
-			_nc_curr_col = 0;
-		    }
-		} else {
-		    if (used != 0)
-			strcat(result, "\n");
-		}
-		if ((bufptr = bufstart) != 0) {
-		    used = strlen(bufptr);
-		    while (iswhite(*bufptr))
-			bufptr++;
-
-		    /*
-		     * Treat a trailing <cr><lf> the same as a <newline> so we
-		     * can read files on OS/2, etc.
-		     */
-		    if ((len = strlen(bufptr)) > 1) {
-			if (bufptr[len - 1] == '\n'
-			    && bufptr[len - 2] == '\r') {
-			    len--;
-			    bufptr[len - 1] = '\n';
-			    bufptr[len] = '\0';
-			}
-		    }
-		} else {
-		    return (EOF);
-		}
-	    } while (bufptr[len - 1] != '\n');	/* complete a line */
-	} while (result[0] == '#');	/* ignore comments */
-    }
-
-    first_column = (bufptr == bufstart);
-
-    _nc_curr_col++;
-    return (*bufptr++);
-}
-
-static void
-push_back(char c)
-/* push a character back onto the input stream */
-{
-    if (bufptr == bufstart)
-	_nc_syserr_abort("Can't backspace off beginning of line");
-    *--bufptr = c;
-}
-
-static long
-stream_pos(void)
-/* return our current character position in the input stream */
-{
-    return (yyin ? ftell(yyin) : (bufptr ? bufptr - bufstart : 0));
-}
-
-static bool
-end_of_stream(void)
-/* are we at end of input? */
-{
-    return ((yyin ? feof(yyin) : (bufptr && *bufptr == '\0'))
-	    ? TRUE : FALSE);
+    FreeAndNull(pushname);
 }
+#endif
