@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2000,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -35,100 +35,108 @@
  * making the output show the indices into the TERMTYPE Strings array.  Doing
  * it that way lets us cut down on the size of the init_keytry() function.
  */
+
+#define USE_TERMLIB 1
 #include <curses.priv.h>
 
-MODULE_ID("$Id: make_keys.c,v 1.7 1999/09/11 17:32:57 Jeffrey.Honig Exp $")
+MODULE_ID("$Id: make_keys.c,v 1.12 2005/08/20 19:58:18 tom Exp $")
 
 #include <names.c>
 
 #define UNKNOWN (SIZEOF(strnames) + SIZEOF(strfnames))
 
-static size_t lookup(const char *name)
+static size_t
+lookup(const char *name)
 {
-	size_t n;
-	bool found = FALSE;
-	for (n = 0; strnames[n] != 0; n++) {
-		if (!strcmp(name, strnames[n])) {
-			found = TRUE;
-			break;
-		}
+    size_t n;
+    bool found = FALSE;
+    for (n = 0; strnames[n] != 0; n++) {
+	if (!strcmp(name, strnames[n])) {
+	    found = TRUE;
+	    break;
 	}
-	if (!found) {
-		for (n = 0; strfnames[n] != 0; n++) {
-			if (!strcmp(name, strfnames[n])) {
-				found = TRUE;
-				break;
-			}
-		}
+    }
+    if (!found) {
+	for (n = 0; strfnames[n] != 0; n++) {
+	    if (!strcmp(name, strfnames[n])) {
+		found = TRUE;
+		break;
+	    }
 	}
-	return found ? n : UNKNOWN;
+    }
+    return found ? n : UNKNOWN;
 }
 
-static void make_keys(FILE *ifp, FILE *ofp)
+static void
+make_keys(FILE *ifp, FILE *ofp)
 {
-	char buffer[BUFSIZ];
-	char from[BUFSIZ];
-	char to[BUFSIZ];
-	int maxlen = 16;
+    char buffer[BUFSIZ];
+    char from[BUFSIZ];
+    char to[BUFSIZ];
+    int maxlen = 16;
 
-	while (fgets(buffer, sizeof(buffer), ifp) != 0) {
-		if (*buffer == '#')
-			continue;
-		if (sscanf(buffer, "%s %s", to, from) == 2) {
-			int code = lookup(from);
-			if (code == UNKNOWN)
-				continue;
-			if ((int)strlen(from) > maxlen)
-				maxlen = strlen(from);
-			fprintf(ofp, "\t{ %4d, %-*.*s },\t/* %s */\n",
-				code,
-				maxlen, maxlen,
-				to,
-				from);
-		}
+    while (fgets(buffer, sizeof(buffer), ifp) != 0) {
+	if (*buffer == '#')
+	    continue;
+	if (sscanf(buffer, "%s %s", to, from) == 2) {
+	    int code = lookup(from);
+	    if (code == UNKNOWN)
+		continue;
+	    if ((int) strlen(from) > maxlen)
+		maxlen = strlen(from);
+	    fprintf(ofp, "\t{ %4d, %-*.*s },\t/* %s */\n",
+		    code,
+		    maxlen, maxlen,
+		    to,
+		    from);
 	}
+    }
 }
 
-static void write_list(FILE *ofp, const char **list)
+static void
+write_list(FILE *ofp, const char **list)
 {
-	while (*list != 0)
-		fprintf(ofp, "%s\n", *list++);
+    while (*list != 0)
+	fprintf(ofp, "%s\n", *list++);
 }
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
-	static const char *prefix[] = {
-		"#ifndef NCU_KEYS_H",
-		"#define NCU_KEYS_H 1",
-		"",
-		"/* This file was generated by MAKE_KEYS */",
-		"",
-		"#ifdef BROKEN_LINKER",
-		"static",
-		"#endif",
-		"struct tinfo_fkeys _nc_tinfo_fkeys[] = {",
-		0
-	};
-	static const char *suffix[] = {
-		"\t{ 0, 0} };",
-		"",
-		"#endif /* NCU_KEYS_H */",
-		0
-	};
+    static const char *prefix[] =
+    {
+	"#ifndef NCU_KEYS_H",
+	"#define NCU_KEYS_H 1",
+	"",
+	"/* This file was generated by MAKE_KEYS */",
+	"",
+	"#if BROKEN_LINKER",
+	"static",
+	"#endif",
+	"struct tinfo_fkeys _nc_tinfo_fkeys[] = {",
+	0
+    };
+    static const char *suffix[] =
+    {
+	"\t{ 0, 0} };",
+	"",
+	"#endif /* NCU_KEYS_H */",
+	0
+    };
 
-	write_list(stdout, prefix);
-	if (argc > 1) {
-		int n;
-		for (n = 1; n < argc; n++) {
-			FILE *fp = fopen(argv[n], "r");
-			if (fp != 0) {
-				make_keys(fp, stdout);
-				fclose(fp);
-			}
-		}
-	} else {
-		make_keys(stdin, stdout);
+    write_list(stdout, prefix);
+    if (argc > 1) {
+	int n;
+	for (n = 1; n < argc; n++) {
+	    FILE *fp = fopen(argv[n], "r");
+	    if (fp != 0) {
+		make_keys(fp, stdout);
+		fclose(fp);
+	    }
 	}
-	write_list(stdout, suffix);
-	return EXIT_SUCCESS;
+    } else {
+	make_keys(stdin, stdout);
+    }
+    write_list(stdout, suffix);
+    return EXIT_SUCCESS;
 }
