@@ -15,13 +15,13 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with TACK; see the file COPYING.  If not, write to
-** the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-** Boston, MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+** Boston, MA 02110-1301, USA
 */
 
 #include <tack.h>
 
-MODULE_ID("$Id: color.c,v 1.5 2004/01/16 23:14:25 Jochen.Voss Exp $")
+MODULE_ID("$Id: color.c,v 1.6 2005/09/17 19:49:16 tom Exp $")
 
 /*
  * Color terminal tests.  Has only one entry point: test_color().
