@@ -15,13 +15,13 @@
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with TACK; see the file COPYING.  If not, write to
-** the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-** Boston, MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+** Boston, MA 02110-1301, USA
 */
 
 #include <tack.h>
 
-MODULE_ID("$Id: fun.c,v 1.2 1999/06/16 00:46:12 tom Exp $")
+MODULE_ID("$Id: fun.c,v 1.6 2005/09/17 19:49:16 tom Exp $")
 
 /*
  * Test the function keys on the terminal.  The code for echo tests
@@ -58,10 +58,6 @@ struct test_list printer_test_list[] = {
 
 #define MAX_STRINGS STRCOUNT
 
-/* scan code externals */
-extern int scan_max;		/* length of longest scan code */
-extern char **scan_up, **scan_down, **scan_name;
-extern int *scan_tested, *scan_length;
 
 /* local definitions */
 static const char *fk_name[MAX_STRINGS];
@@ -102,7 +98,7 @@ keys_tested(
 			if (fk_label[i]) {
 				sprintf(outbuf, "%s %s",
 					fk_name[i] ? fk_name[i] : "??", fk_label[i]);
-				put_columns(outbuf, strlen(outbuf), 16);
+				put_columns(outbuf, (int) strlen(outbuf), 16);
 			}
 		}
 		put_newlines(2);
@@ -215,7 +211,7 @@ enter_key(
 
 static void
 fresh_line(void)
-{				/* clear the line for a new fumction key line */
+{				/* clear the line for a new function key line */
 	if (over_strike) {
 		put_crlf();
 	} else {
@@ -610,7 +606,7 @@ funkey_prog(
 		mm[i] = '\0';
 		put_crlf();
 		if (mm[0] != 'a' || mm[1] != 'b' || mm[2] != 'c') {
-			sprintf(temp, "Error string recieved was: %s", expand(mm));
+			sprintf(temp, "Error string received was: %s", expand(mm));
 			ptextln(temp);
 		} else {
 			putln("Thank you\n");
@@ -764,8 +760,8 @@ report_help(int crx)
 {
 	ptextln("The following commands may also be entered:");
 	ptextln(" clear   clear screen.");
-	ptextln(" columns print a test patterm to help count screen width.");
-	ptextln(" lines   print a test patterm to help count screen length.");
+	ptextln(" columns print a test pattern to help count screen width.");
+	ptextln(" lines   print a test pattern to help count screen length.");
 	ptextln(" end     exit.");
 	ptextln(" echo    redisplay last report.");
 	if (crx) {
